from math import *
from general import *
import sys,getopt,pywt
from scipy.io.matlab import *
sys.path.append('readin')
sys.path.append('wavelet')
sys.path.append('struct')
sys.path.append('word')
sys.path.append('logistic_regression')
from plr import *
from get_seq import *
from wenergy import *
from DNAstruct import *
from wordcount import *
import numpy as np
import time

##################
#   class Seq    #
##################

class Region:
    def __init__(self,window_size,chr_id,bpstart,bpend,overlap='257'):
        self.bpstart=int(bpstart)
        self.bpend=int(bpend)
        self.chr_id=chr_id
        self.window_size=int(window_size)
        self.overlap=int(overlap)
        
    def __str__(self):
        return self.chr_id+'_'+str(self.bpstart)+'_'+str(self.bpend)
    
    def centers(self):
        half_window=self.window_size/2
        return np.arange(self.bpstart+half_window,self.bpend-half_window,self.overlap)


class Seq:
    def __init__(self,posfile,seqlen,fd,region=None):
        self.posfile = posfile
        self.seqlen = seqlen
        self.region=region
        if posfile:
            self.seqfile = posfile.split('/')[-1]
        else:
            self.seqfile = str(region)
        self.seq_dicode = []
        self.seq = []
        self.fasta_dir = fd
        
    getseq = get_seq # readin seq according to position file
    Nseq2wt_allmarker = Nseq2wt_allmarker # wavelet energy 
    Nseq2struct_allmarker = Nseq2struct_allmarker # DNA property 


##################
#  main function #
##################

def usage():
    print """
    command -m model_file -r chr_id:start:end:overlap -g genome_directory -o output 
    """
def main():
    #print sys.argv
    ####################################
    # get parameters from command line #
    ####################################
    try:
        #opts, args = getopt.getopt(sys.argv[1:], "p:n:t:l:o:s:", ["positive_file=","negative_file=","test_file=","seqlen=","output=","species="])
        opts, args = getopt.getopt(sys.argv[1:], "m:r:g:o:", ["model_file=","region=","genome_directory=","output=",])
        
    except getopt.GetoptError:
        # print help information and exit:
        usage()
        sys.exit(2)
    if len(sys.argv)<6:
        usage()
        sys.exit()
    testpos = None
    output =''

    #print opts, args
    for o, a in opts:
        if o in ("-m", "--model_file"):
             model_file = a
               
             if model_file == None:
                 usage()
                 sys.exit()
             else:
                 print "\nLoading Model Information"
                 model_info=np.load(model_file)
                 model=model_info['model']
                 select_idx=model_info['select_idx']
                 seqlen=model_info['seqlen']
                 model_name=str(model_info['name'])
                 print 'Model name:',model_name
          
                 
        elif o in ("-o", "--output"):
             output = a


        elif o in ("-r","--region"):
             
             region=a
             if  region== None:
                 usage()
                 sys.exit()
             else:
                fields =region.split(':')
                region=Region(seqlen,*fields)
                print '\nInput region:',region,'covered by:',len(region.centers()),' windows of length:',region.window_size,'overlapped by:',region.overlap

        elif o in ('-g','--genome_directory'):
             genome_directory=a

             if genome_directory == None:
                 usage()
                 sys.exit()    

        else:
             assert False, "unhandled option"

    fasta_dir = genome_directory # its better using "\\" since '\' sometimes gives error message, for example, '\t' represent other meanings.  
    

    ########################
    #     get sequence     #
    ########################
    print "\nStepI: readin sequences"
    test = Seq(testpos, seqlen,fasta_dir,region)
    test.getseq(region)
    #np.savetxt(output+'_testseq.txt',test.seq, delimiter="\t",fmt = '%s')

    #####################
    #   wenergy         #
    #####################
    print "\nStepII: calculate wavelet energy"
    wenergy_test = test.Nseq2wt_allmarker(output)

    #####################
    #   DNA property    #
    #####################
    print "\nStepIII: calculate DNA-property"
    struct_test = test.Nseq2struct_allmarker(output)
    #####################
    #     word count    #
    #####################
    print "\nStep IV: word count"
    word_test= seq2mtx_weng_allmarker_test(test.seq,select_idx)
    ####################
    #  save txt file   #
    ####################
    print "\nStep V: logistic regression"
    val_test = np.hstack((wenergy_test,word_test,struct_test))
    
    test_score= np.squeeze(np.asarray(predict(val_test,model[0],ptype = 'response')))
    np.savetxt(output+model_name+'_'+str(region)+'_score.txt',np.vstack((region.centers(),test_score)).T, delimiter="\t",fmt = '%.8e')
    #savemat(output+'val_test', {'val_test': val_test},oned_as='row')
if __name__ == "__main__":
    t1=time.clock()
    main()
    print '\nTime elapsed:',time.clock()-t1



   



